@@ -143,51 +143,212 @@ release_engines (GNOMESpellDictionary *dict)
 	dict->changed = TRUE;
 }
 
-#define KNOWN_LANGUAGES 42
-static gchar *known_languages [KNOWN_LANGUAGES*2 + 1] = {
-	"br", N_("Breton"),
-	"ca", N_("Catalan"),
-	"cs", N_("Czech"),
-	"cy", N_("Welsh"),
-	"da", N_("Danish"),
-	"de-AT", N_("German (Austria)"),
-	"de-DE", N_("German (Germany)"),
-	"de-CH", N_("German (Swiss)"),
-	"el", N_("Greek"),
-	"en-US", N_("English (American)"),
-	"en-GB", N_("English (British)"),
-	"en-CA", N_("English (Canadian)"),
-	"eo", N_("Esperanto"),
-	"es", N_("Spanish"),
-	"fi", N_("Finnish"),
-	"fo", N_("Faroese"),
-	"fr-FR", N_("French (France)"),
-	"fr-CH", N_("French (Swiss)"),
-	"ga", N_("Irish"),
-	"gl", N_("Galician"),
-	"he", N_("Hebrew"),
-	"hu", N_("Hungarian"),
-	"it", N_("Italian"),
-	"is", N_("Icelandic"),
-	"la", N_("Latin"),
-	"lt", N_("Lithuanian"),
-	"nb-NO", N_("Norwegian (Bokmal)"),
-	"nl", N_("Dutch"),
-	"nn-NO", N_("Norwegian (Nyorsk)"),
-	"no", N_("Norwegian"),
-	"pl", N_("Polish"),
-	"pt-PT", N_("Portuguese (Portugal)"),
-	"pt-BR", N_("Portuguese (Brazilian)"),
-	"ro", N_("Romanian"),
-	"ru", N_("Russian"),
-	"sc", N_("Sardinian"),
-	"sk", N_("Slovak"),
-	"sl", N_("Slovenian"),
-	"sv", N_("Swedish"),
-	"tr", N_("Turkish"),
-	"uk", N_("Ukrainian"),
-	"yi", N_("Yiddish"),
-	NULL
+static LangInfo known_languages [] = {
+/* check <http://aspell.sourceforge.net/man-html/Supported.html> and
+              <http://aspell.sourceforge.net/man-html/Unsupported.html>.
+   short language codes according to ISO 639-1. */
+	{"aa", N_("Afar")},
+	{"af", N_("Afrikaans")},
+	{"ak", N_("Akan")},
+	{"am", N_("Amharic")},
+	{"ar", N_("Arabic")},
+	{"as", N_("Assamese")},
+	{"ast", N_("Asturian")},
+	{"av", N_("Avar")},
+	{"ay", N_("Aymara")},
+	{"az", N_("Azerbaijani")},
+	{"ba", N_("Bashkir")},
+	{"ban", N_("Balinese")},
+	{"be", N_("Belarusian")},
+	{"bem", N_("Bemba")},
+	{"bg", N_("Bulgarian")},
+	{"bh", N_("Bihari")},
+	{"bi", N_("Bislama")},
+	{"bm", N_("Bambara")},
+	{"bn", N_("Bengali")},
+	{"bo", N_("Tibetan")},
+	{"br", N_("Breton")},
+	{"bs", N_("Bosnian")},
+	{"ca", N_("Catalan")},
+	{"ce", N_("Chechen")},
+	{"ceb", N_("Cebuano")},
+	{"ch", N_("Chamorro")},
+	{"chk", N_("Chuukese")},
+	{"co", N_("Corsican")},
+	{"cs", N_("Czech")},
+	{"csb", N_("Kashubian")},
+	{"cv", N_("Chuvash")},
+	{"cy", N_("Welsh")},
+	{"da", N_("Danish")},
+	{"de-AT", N_("German (Austria)")},
+	{"de-DE", N_("German (Germany)")},
+	{"de-CH", N_("German (Swiss)")},
+	{"ee", N_("Ewe")},
+	{"el", N_("Greek")},
+	{"en-US", N_("English (American)")},
+	{"en-GB", N_("English (British)")},
+	{"en-CA", N_("English (Canadian)")},
+	{"eo", N_("Esperanto")},
+	{"es", N_("Spanish")},
+	{"et", N_("Estonian")},
+	{"eu", N_("Basque")},
+	{"fa", N_("Farsi")},
+	{"ff", N_("Fulah")},
+	{"fi", N_("Finnish")},
+	{"fj", N_("Fijian")},
+	{"fo", N_("Faroese")},
+	{"fr-FR", N_("French (France)")},
+	{"fr-CH", N_("French (Swiss)")},
+	{"fur", N_("Friulian")},
+	{"fy", N_("Frisian")},
+	{"ga", N_("Irish")},
+	{"gd", N_("Scottish Gaelic")},
+	{"gl", N_("Galician")},
+	{"gn", N_("Guarani")},
+	{"gu", N_("Gujarati")},
+	{"gv", N_("Manx Gaelic")},
+	{"ha", N_("Hausa")},
+	{"haw", N_("Hawaiian")},
+	{"he", N_("Hebrew")},
+	{"hi", N_("Hindi")},
+	{"hil", N_("Hiligaynon")},
+	{"ho", N_("Hiri Motu")},
+	{"hr", N_("Croatian")},
+	{"hsb", N_("Upper Sorbian")},
+	{"ht", N_("Haitian Creole")},
+	{"hu", N_("Hungarian")},
+	{"hy", N_("Armenian")},
+	{"hz", N_("Herero")},
+	{"ia", N_("Interlingua")},
+	{"iba", N_("Iban")},
+	{"id", N_("Indonesian")},
+	{"ig", N_("Igbo")},
+	{"ii", N_("Sichuan Yi")},
+	{"ilo", N_("Iloko")},
+	{"io", N_("Ido")},
+	{"is", N_("Icelandic")},
+	{"it", N_("Italian")},
+	{"ja", N_("Japanese")},
+	{"jv", N_("Javanese")},
+	{"ka", N_("Georgian")},
+	{"kac", N_("Kachin")},
+	{"kg", N_("Kongo")},
+	{"kha", N_("Khasi")},
+	{"ki", N_("Kikuyu")},
+	{"kj", N_("Kwanyama")},
+	{"kk", N_("Kazakh")},
+	{"kl", N_("Kalaallisut / Greenlandic")},
+	{"km", N_("Khmer")},
+	{"kn", N_("Kannada")},
+	{"ko", N_("Korean")},
+	{"kok", N_("Konkani")},
+	{"kr", N_("Kanuri")},
+	{"ks", N_("Kashmiri")},
+	{"ku", N_("Kurdish")},
+	{"kv", N_("Komi")},
+	{"kw", N_("Cornish")},
+	{"ky", N_("Kirghiz")},
+	{"la", N_("Latin")},
+	{"lb", N_("Luxembourgish")},
+	{"lg", N_("Ganda")},
+	{"li", N_("Limburgian")},
+	{"ln", N_("Lingala")},
+	{"lo", N_("Lao")},
+	{"loz", N_("Lozi")},
+	{"lt", N_("Lithuanian")},
+	{"lu", N_("Luba-Katanga")},
+	{"luo", N_("Luo")},
+	{"lv", N_("Latvian")},
+	{"mg", N_("Malagasy")},
+	{"mh", N_("Marshallese")},
+	{"mi", N_("Maori")},
+	{"min", N_("Minangkabau")},
+	{"mk", N_("Macedonian")},
+	{"ml", N_("Malayalam")},
+	{"mn", N_("Mongolian")},
+	{"mo", N_("Moldavian")},
+	{"mr", N_("Marathi")},
+	{"ms", N_("Malay")},
+	{"mt", N_("Maltese")},
+	{"my", N_("Burmese")},
+	{"nb-NO", N_("Norwegian (Bokmal)")},
+	{"nd", N_("North Ndebele")},
+	{"nds", N_("Low Saxon")},
+	{"ne", N_("Nepali")},
+	{"ng", N_("Ndonga")},
+	{"niu", N_("Niuean")},
+	{"nl", N_("Dutch")},
+	{"nn-NO", N_("Norwegian (Nyorsk)")},
+	{"no", N_("Norwegian")},
+	{"nr", N_("South Ndebele")},
+	{"nso", N_("Northern Sotho")},
+	{"nv", N_("Navajo")},
+	{"ny", N_("Nyanja/Chichewa")},
+	{"oc", N_("Occitan")},
+	{"om", N_("Oromo")},
+	{"or", N_("Oriya")},
+	{"os", N_("Ossetic")},
+	{"pa", N_("Punjabi")},
+	{"pam", N_("Pampanga")},
+	{"pap", N_("Papiamento")},
+	{"pl", N_("Polish")},
+	{"ps", N_("Pushto")},
+	{"pt-PT", N_("Portuguese (Portugal)")},
+	{"pt-BR", N_("Portuguese (Brazilian)")},
+	{"qu", N_("Quechua")},
+	{"rar", N_("Rarotongan")},
+	{"rn", N_("Rundi")},
+	{"ro", N_("Romanian")},
+	{"ru", N_("Russian")},
+	{"rw", N_("Kinyarwanda")},
+	{"sc", N_("Sardinian")},
+	{"sd", N_("Sindhi")},
+	{"se", N_("Northern Sami")},
+	{"sg", N_("Sango")},
+	{"si", N_("Sinhalese")},
+	{"sk", N_("Slovak")},
+	{"sl", N_("Slovenian")},
+	{"sm", N_("Samoan")},
+	{"sn", N_("Shona")},
+	{"so", N_("Somali")},
+	{"sq", N_("Albanian")},
+	{"sr", N_("Serbian")},
+	{"ss", N_("Swati")},
+	{"st", N_("Southern Sotho")},
+	{"su", N_("Sundanese")},
+	{"sv", N_("Swedish")},
+	{"sw", N_("Swahili")},
+	{"ta", N_("Tamil")},
+	{"te", N_("Telugu")},
+	{"tet", N_("Tetum")},
+	{"tg", N_("Tajik")},
+	{"th", N_("Thai")},
+	{"ti", N_("Tigrinya")},
+	{"tk", N_("Turkmen")},
+	{"tkl", N_("Tokelau")},
+	{"tl", N_("Tagalog")},
+	{"tn", N_("Tswana")},
+	{"to", N_("Tonga")},
+	{"tpi", N_("Tok Pisin")},
+	{"tr", N_("Turkish")},
+	{"ts", N_("Tsonga")},
+	{"tt", N_("Tatar")},
+	{"tw", N_("Twi")},
+	{"ty", N_("Tahitian")},
+	{"ug", N_("Uighur")},
+	{"uk", N_("Ukrainian")},
+	{"ur", N_("Urdu")},
+	{"uz", N_("Uzbek")},
+	{"ve", N_("Venda")},
+	{"vi", N_("Vietnamese")},
+	{"wa", N_("Walloon")},
+	{"wo", N_("Wolof")},
+	{"xh", N_("Xhosa")},
+	{"yi", N_("Yiddish")},
+	{"yo", N_("Yoruba")},
+	{"za", N_("Zhuang")},
+	{"zh", N_("Chinese")},
+	{"zu", N_("Zulu")}
 };
 
 static GSList *
@@ -203,16 +364,15 @@ get_languages_real (gint *ln)
 
 	langs = NULL;
 	*ln = 0;
-	for (i=0; known_languages [i]; i++) {
+	for (i=0; i < G_N_ELEMENTS (known_languages); i++) {
 		config = new_aspell_config ();
-		aspell_config_replace (config, "language-tag", known_languages [i]);
-		i++;
+		aspell_config_replace (config, "language-tag", known_languages [i].abbreviation);
 		err = new_aspell_speller (config);
 		if (aspell_error_number (err) == 0) {
 			speller = to_aspell_speller (err);
-			DICT_DEBUG (printf ("Language: %s\n", known_languages [i]));
+			DICT_DEBUG (printf ("Language: %s\n", known_languages [i].name));
 			delete_aspell_speller (speller);
-			langs = g_slist_prepend (langs, GINT_TO_POINTER (i - 1));
+			langs = g_slist_prepend (langs, GINT_TO_POINTER (i));
 			(*ln) ++;
 		}
 	}
@@ -254,7 +414,7 @@ get_languages (gint *ln)
 	mtime = gconf_client_get_int (gc, GNOME_SPELL_GCONF_DIR "/mtime", NULL);
 	kl = gconf_client_get_int (gc, GNOME_SPELL_GCONF_DIR "/known_languages", NULL);
 
- 	if (stat (ASPELL_DICT, &buf) || buf.st_mtime != mtime || kl != KNOWN_LANGUAGES) {
+	if (stat (ASPELL_DICT, &buf) || buf.st_mtime != mtime || kl != G_N_ELEMENTS(known_languages)) {
 		GString *str;
 		langs = get_languages_real (ln);
 
@@ -266,7 +426,7 @@ get_languages (gint *ln)
 			l = l->next;
 		}
 		gconf_client_set_int (gc, GNOME_SPELL_GCONF_DIR "/mtime", buf.st_mtime, NULL);
-		gconf_client_set_int (gc, GNOME_SPELL_GCONF_DIR "/known_languages", KNOWN_LANGUAGES, NULL);
+		gconf_client_set_int (gc, GNOME_SPELL_GCONF_DIR "/known_languages", G_N_ELEMENTS(known_languages), NULL);
 		g_string_free (str, TRUE);
 		gnome_config_sync ();
 	} else
@@ -298,8 +458,8 @@ impl_gnome_spell_dictionary_get_languages (PortableServer_Servant servant, CORBA
 	for (i = ln - 1, l = langs; l; l = l->next, i--) {
 
 		pos = GPOINTER_TO_INT (l->data);
-		seq->_buffer [i].name = CORBA_string_dup (_(known_languages [pos + 1]));
-		seq->_buffer [i].abbreviation = CORBA_string_dup (known_languages [pos]);
+		seq->_buffer [i].name = CORBA_string_dup (_(known_languages [pos].name));
+		seq->_buffer [i].abbreviation = CORBA_string_dup (known_languages [pos].abbreviation);
 	}
 	CORBA_sequence_set_release (seq, CORBA_TRUE);
 	g_slist_free (langs);
